<?php

if (!defined('_PS_VERSION_')) {
	exit;
}

require_once __DIR__ . '/api-client/vendor/autoload.php';

class minifaktura extends Module {

	const CUSTOM_ID_PREFIX		 = 'presta';
	const PAYMENT_MAP_PREFIX	 = 'MF_payment_';
	const DELIVERY_MAP_PREFIX	 = 'MF_delivery_';
	const STATUS_MAP_PREFIX		 = 'MF_status_';
	const ONLY_ORDER					 = 'order';
	const ORDER_INVOICE				 = 'invoice';

	private $apikey;
	private $apiSecret;
	private $apiDomain;
	private $invoice;
	private $paymentMapping	 = array(
			'transfer'				 => array(
					'id'	 => 'transfer',
					'name' => 'Transfer'
			),
			'cash'						 => array(
					'id'	 => 'cash',
					'name' => 'Cash'
			),
			'cash_on_delivery' => array(
					'id'	 => 'cash_on_delivery',
					'name' => 'Cash on delivery'
			),
			'paypal'					 => array(
					'id'	 => 'paypal',
					'name' => 'Paypal'
			),
			'trustpay'				 => array(
					'id'	 => 'trustpay',
					'name' => 'Trustpay'
			),
			'credit_card'			 => array(
					'id'	 => 'credit_card',
					'name' => 'Credit card'
			),
			'advance_payment'	 => array(
					'id'	 => 'advance_payment',
					'name' => 'Advance payment'
			),
			'direct_debit'		 => array(
					'id'	 => 'direct_debit',
					'name' => 'Direct debit'
			),
			'other'						 => array(
					'id'	 => 'other',
					'name' => 'Other'
			),
			'inapp'						 => array(
					'id'	 => 'inapp',
					'name' => 'inapp'
			),
			'gplay'						 => array(
					'id'	 => 'gplay',
					'name' => 'gplay'
			)
	);
	private $deliveryMapping = array(
			'none'						 => array(
					'id'	 => 'none',
					'name' => 'None'
			),
			'mail'						 => array(
					'id'	 => 'mail',
					'name' => 'Mail'
			),
			'cash'						 => array(
					'id'	 => 'cash',
					'name' => 'Courier'
			),
			'personal_pickup'	 => array(
					'id'	 => 'personal_pickup',
					'name' => 'Personal pickup'
			),
			'freight_service'	 => array(
					'id'	 => 'freight_service',
					'name' => 'Freight service'
			)
	);
	private $statusMapping	 = array(
			'pending_processment'	 => array(
					'id'	 => 'pending_processment',
					'name' => 'Pending processment'
			),
			'processing'					 => array(
					'id'	 => 'processing',
					'name' => 'Processing'
			),
			'sent'								 => array(
					'id'	 => 'sent',
					'name' => 'Sent'
			),
			'pending_wares'				 => array(
					'id'	 => 'pending_wares',
					'name' => 'Pending wares'
			),
			'pending_payment'			 => array(
					'id'	 => 'pending_payment',
					'name' => 'Pending payment'
			),
			'payment_received'		 => array(
					'id'	 => 'payment_received',
					'name' => 'Payment received'
			),
			'canceled'						 => array(
					'id'	 => 'canceled',
					'name' => 'Canceled'
			)
	);

	public function __construct() {
		$this->name					 = "minifaktura";
		$this->tab					 = "billing_invoicing";
		$this->version			 = 1.0;
		$this->author				 = "www.minifaktura.sk";
		$this->need_instance = 1;

		parent::__construct();

		$this->displayName = $this->l('miniFaktura');
		$this->description = $this->l('www.minifaktura.sk API for PrestaShop 1.5 and 1.6');

		$config					 = Configuration::getMultiple(array('MINIFAKTURA_APIKEY', 'MINIFAKTURA_APISECRET', 'MINIFAKTURA_APIDOMAIN', 'MINIFAKTURA_INVOICE'));
		$this->apikey		 = $config['MINIFAKTURA_APIKEY'];
		$this->apiSecret = $config['MINIFAKTURA_APISECRET'];
		$this->apiDomain = $config['MINIFAKTURA_APIDOMAIN'];
		$this->invoice	 = $config['MINIFAKTURA_INVOICE'];

		$this->warning = "";
		if (empty($config['MINIFAKTURA_APIKEY'])) {
			$this->warning .= $this->l('API key must be filled.') . " ";
		}
		if (empty($config['MINIFAKTURA_APISECRET'])) {
			$this->warning .= $this->l('API secret must be filled.');
		}
		if (empty($config['MINIFAKTURA_APIDOMAIN'])) {
			$this->warning .= $this->l('API domain must be filled.');
		}
	}

	public function install() {
		if (Shop::isFeatureActive()) {
			Shop::setContext(Shop::CONTEXT_ALL);
		}

		return parent::install() && Configuration::updateValue('MINIFAKTURA_APIKEY', '') && Configuration::updateValue('MINIFAKTURA_APISECRET', '') && Configuration::updateValue('MINIFAKTURA_APIDOMAIN', '') && Configuration::updateValue('MINIFAKTURA_INVOICE', FALSE) && $this->registerHook('actionValidateOrder') && $this->registerHook('actionOrderStatusPostUpdate') && $this->registerHook('actionPaymentConfirmation');
	}

	public function uninstall() {
		foreach (PaymentModule::getInstalledPaymentModules() as $paymentModule) {
			$index = self::PAYMENT_MAP_PREFIX . $paymentModule['name'];
			Configuration::deleteByName($index);
		}
		$default_lang = (int) Configuration::get('PS_LANG_DEFAULT');
		foreach (Carrier::getCarriers($default_lang) as $carrier) {
			$index = self::DELIVERY_MAP_PREFIX . $carrier['id_carrier'];
			$value = Configuration::deleteByName($index);
		}
		foreach (OrderState::getOrderStates($default_lang) as $state) {
			$index = self::STATUS_MAP_PREFIX . $state['id_order_state'];
			Configuration::deleteByName($index);
		}
		return parent::uninstall() && Configuration::deleteByName('MINIFAKTURA_APIKEY') && Configuration::deleteByName('MINIFAKTURA_APISECRET') && Configuration::deleteByName('MINIFAKTURA_APIDOMAIN') && Configuration::deleteByName('MINIFAKTURA_INVOICE');
	}

	public function getContent() {
		$output = "";

		if (Tools::isSubmit('submit' . $this->name)) {
			$apikey		 = strval(Tools::getValue('MINIFAKTURA_APIKEY'));
			$apiSecret = strval(Tools::getValue('MINIFAKTURA_APISECRET'));
			$apiDomain = strval(Tools::getValue('MINIFAKTURA_APIDOMAIN'));
			$invoice	 = strval(Tools::getValue('MINIFAKTURA_INVOICE'));
			if (empty($apikey)) {
				$output .= $this->displayError($this->l('Invalid minifaktura API KEY'));
			}
			if (empty($apiSecret)) {
				$output .= $this->displayError($this->l('Invalid minifaktura API secret'));
			}
			if (empty($apiDomain)) {
				$output .= $this->displayError($this->l('Invalid minifaktura API domain'));
			}
			Configuration::updateValue('MINIFAKTURA_APIKEY', $apikey);
			Configuration::updateValue('MINIFAKTURA_APISECRET', $apiSecret);
			Configuration::updateValue('MINIFAKTURA_APIDOMAIN', $apiDomain);
			Configuration::updateValue('MINIFAKTURA_INVOICE', $invoice);

			foreach (PaymentModule::getInstalledPaymentModules() as $paymentModule) {
				$index = self::PAYMENT_MAP_PREFIX . $paymentModule['name'];
				$value = strval(Tools::getValue($index));
				if (!empty($value)) {
					Configuration::updateValue($index, $value);
				}
			}

			$default_lang = (int) Configuration::get('PS_LANG_DEFAULT');

			foreach (OrderState::getOrderStates($default_lang) as $state) {
				$index = self::STATUS_MAP_PREFIX . $state['id_order_state'];
				$value = strval(Tools::getValue($index));
				if (!empty($value)) {
					Configuration::updateValue($index, $value);
				}
			}

			foreach (Carrier::getCarriers($default_lang) as $carrier) {
				$index = self::DELIVERY_MAP_PREFIX . $carrier['id_carrier'];
				$value = strval(Tools::getValue($index));
				if (!empty($value)) {
					Configuration::updateValue($index, $value);
				}
			}

			$output .= $this->displayConfirmation($this->l('Settings updated'));
		}

		return $output . $this->displayForm();
	}

	public function displayForm() {
		$default_lang = (int) Configuration::get('PS_LANG_DEFAULT');

		$fields_form	 = array();
		$fields_form[] = array(
				'form' => array(
						'legend' => array(
								'title' => $this->l('miniFaktura settings')
						),
						'input'	 => array(
								array(
										'type'		 => 'text',
										'label'		 => $this->l('minifaktura APIKEY'),
										'name'		 => 'MINIFAKTURA_APIKEY',
										'size'		 => 50,
										'required' => true
								),
								array(
										'type'		 => 'text',
										'label'		 => $this->l('minifaktura API secret'),
										'name'		 => 'MINIFAKTURA_APISECRET',
										'size'		 => 50,
										'required' => true
								),
								array(
										'type'		 => 'text',
										'label'		 => $this->l('minifaktura API domain'),
										'name'		 => 'MINIFAKTURA_APIDOMAIN',
										'size'		 => 50,
										'required' => true
								),
								array(
										'type'		 => 'select',
										'label'		 => 'Creations',
										'name'		 => 'MINIFAKTURA_INVOICE',
										'options'	 => array(
												'query'	 => array(
														array(
																'id'	 => self::ONLY_ORDER,
																'name' => 'Create order only'
														),
														array(
																'id'	 => self::ORDER_INVOICE,
																'name' => 'Create order and invoice'
														)
												),
												'id'		 => 'id',
												'name'	 => 'name'
										)
								)
						)
				)
		);

		$paymentInput		 = array();
		$paymentModules	 = PaymentModule::getInstalledPaymentModules();
		foreach ($paymentModules as $paymentModule) {
			$paymentInput[] = array(
					'type'		 => 'select',
					'label'		 => $paymentModule['name'],
					'name'		 => self::PAYMENT_MAP_PREFIX . $paymentModule['name'],
					'options'	 => array(
							'query'	 => $this->paymentMapping,
							'id'		 => 'id',
							'name'	 => 'name'
					)
			);
		}
		$fields_form[] = array(
				'form' => array(
						'legend' => array(
								'title' => $this->l('Payment mapping')
						),
						'input'	 => $paymentInput
				)
		);

		$deliveryInput = array();
		$carriers			 = Carrier::getCarriers($default_lang);
		foreach ($carriers as $carrier) {
			$deliveryInput[] = array(
					'type'		 => 'select',
					'label'		 => $carrier['name'],
					'name'		 => self::DELIVERY_MAP_PREFIX . $carrier['id_carrier'],
					'options'	 => array(
							'query'	 => $this->deliveryMapping,
							'id'		 => 'id',
							'name'	 => 'name'
					)
			);
		}
		$fields_form[] = array(
				'form' => array(
						'legend' => array(
								'title' => $this->l('Delivery mapping')
						),
						'input'	 => $deliveryInput
				)
		);

		$statusInput = array();
		$orderStates = OrderState::getOrderStates($default_lang);
		foreach ($orderStates as $state) {
			$statusInput[] = array(
					'type'		 => 'select',
					'label'		 => $state['name'],
					'name'		 => self::STATUS_MAP_PREFIX . $state['id_order_state'],
					'options'	 => array(
							'query'	 => $this->statusMapping,
							'id'		 => 'id',
							'name'	 => 'name'
					)
			);
		}
		$fields_form[] = array(
				'form' => array(
						'legend' => array(
								'title' => $this->l('Statuses mapping')
						),
						'input'	 => $statusInput
				)
		);

		$fields_form[] = array(
				'form' => array(
						'submit' => array(
								'title'	 => $this->l('Save'),
								'class'	 => 'button'
						)
				)
		);

		$helper = new HelperForm();

		$helper->module					 = $this;
		$helper->name_controller = $this->name;
		$helper->token					 = Tools::getAdminTokenLite('AdminModules');
		$helper->currentIndex		 = AdminController::$currentIndex . '&configure=' . $this->name;

		$helper->default_form_language		 = $default_lang;
		$helper->allow_employee_form_lang	 = $default_lang;

		$helper->title				 = $this->displayName;
		$helper->show_toolbar	 = false;
		$helper->submit_action = 'submit' . $this->name;

		$config = Configuration::getMultiple(array('MINIFAKTURA_APIKEY', 'MINIFAKTURA_APISECRET', 'MINIFAKTURA_APIDOMAIN', 'MINIFAKTURA_INVOICE'));

		$helper->fields_value['MINIFAKTURA_APIKEY']		 = $config['MINIFAKTURA_APIKEY'];
		$helper->fields_value['MINIFAKTURA_APISECRET'] = $config['MINIFAKTURA_APISECRET'];
		$helper->fields_value['MINIFAKTURA_APIDOMAIN'] = $config['MINIFAKTURA_APIDOMAIN'];
		$helper->fields_value['MINIFAKTURA_INVOICE']	 = $config['MINIFAKTURA_INVOICE'];
		foreach ($paymentModules as $paymentModule) {
			$index = self::PAYMENT_MAP_PREFIX . $paymentModule['name'];
			$value = Configuration::get($index);
			if (!empty($value)) {
				$helper->fields_value[$index] = $value;
			}
		}
		foreach ($carriers as $carrier) {
			$index = self::DELIVERY_MAP_PREFIX . $carrier['id_carrier'];
			$value = Configuration::get($index);
			if (!empty($value)) {
				$helper->fields_value[$index] = $value;
			}
		}
		foreach ($orderStates as $state) {
			$index = self::STATUS_MAP_PREFIX . $state['id_order_state'];
			$value = Configuration::get($index);
			if (!empty($value)) {
				$helper->fields_value[$index] = $value;
			}
		}

		return $helper->generateForm($fields_form);
	}

	private function getDocumentData($cart, $order) {
		$items = array();

		foreach ($cart->getProducts() as $product) {
			if (!empty($product['reference'])) {
				$stockNumber = $product['reference'];
			} else if (!empty($product['ean13'])) {
				$stockNumber = $product['ean13'];
			} else if (!empty($product['upc'])) {
				$stockNumber = $product['upc'];
			} else {
				$stockNumber = NULL;
			}
			$items[] = array(
					'label'				 => $product['name'] . (empty($product['attributes']) ? NULL : (' - ' . $product['attributes'])),
					'count'				 => $product['quantity'],
					'price'				 => $product['price'],
					'tax'					 => $product['rate'],
					'unit'				 => $product['unity'],
					'stockNumber'	 => $stockNumber
			);
		}

		$customer				 = new Customer($cart->id_customer);
		$invoiceAddress	 = new Address($cart->id_address_invoice);
		$invoiceState		 = new State($invoiceAddress->id_state);
		$deliveryAddress = new Address($cart->id_address_delivery);
		$deliveryState	 = new State($deliveryAddress->id_state);
		$carrier				 = new Carrier($order->id_carrier);
		$currency				 = new Currency($order->id_currency);
		$payment				 = Configuration::get(self::PAYMENT_MAP_PREFIX . $order->module);
		$delivery				 = Configuration::get(self::DELIVERY_MAP_PREFIX . $order->id_carrier);
		
		$items[] = $carrier->is_free ? array(
				'label'	 => $carrier->name,
				'count'	 => 1,
				'price'	 => 0,
				'tax'		 => 0
						) : array(
				'label'	 => $carrier->name,
				'count'	 => 1,
				'price'	 => $order->total_shipping_tax_excl,
				'tax'		 => $order->carrier_tax_rate
		);

		return array(
				'client'			 => array(
						'email'						 => $customer->email,
						'company'					 => $invoiceAddress->company,
						'name'						 => $invoiceAddress->firstname,
						'surname'					 => $invoiceAddress->lastname,
						'street'					 => $invoiceAddress->address1,
						'street2'					 => $invoiceAddress->address2,
						'zip'							 => $invoiceAddress->postcode,
						'city'						 => $invoiceAddress->city,
						'province'				 => $invoiceState->name,
						'country'					 => $invoiceAddress->country,
						'shippingCompany'	 => $deliveryAddress->company,
						'shippingName'		 => $deliveryAddress->firstname,
						'shippingSurname'	 => $deliveryAddress->lastname,
						'shippingStreet'	 => $deliveryAddress->address1,
						'shippingStreet2'	 => $deliveryAddress->address2,
						'shippingZip'			 => $deliveryAddress->postcode,
						'shippingCity'		 => $deliveryAddress->city,
						'shippingProvince' => $deliveryState->name,
						'shippingCountry'	 => $deliveryAddress->country,
						'vatId'						 => $invoiceAddress->vat_number,
						'phone'						 => $invoiceAddress->phone,
						'mobil'						 => $invoiceAddress->phone_mobile
				),
				'items'				 => $items,
				'customId'		 => self::CUSTOM_ID_PREFIX . $order->id,
				'payment'			 => $this->getPaymentType($payment),
				'deliveryType' => $this->getDeliveryType($delivery),
				'currency'		 => $currency->iso_code
		);
	}

	public function hookActionValidateOrder($params) {
		$api = new \iInvoices\Api\ApiClient($this->apiDomain, $this->apikey, $this->apiSecret);

		$cart	 = $params['cart'];
		$order = $params['order'];

		$data = $this->getDocumentData($cart, $order);

		try {
			$api->orders->create($data);
		} catch (\Exception $e) {
			Tools::dieOrLog(sprintf($this->l('Error: minifaktura save order: %s  data: %s'), $e->getMessage(), json_encode($data)), FALSE);
		}
		if ($this->invoice === self::ORDER_INVOICE) {
			try {
				$api->invoices->create($data);
			} catch (\Exception $e) {
				Tools::dieOrLog(sprintf($this->l('Error: minifaktura save invoice: %s  data: %s'), $e->getMessage(), json_encode($data)), FALSE);
			}
		}
	}

	private function getPaymentType($payment) {
		if (array_key_exists($payment, $this->paymentMapping)) {
			return $this->paymentMapping[$payment]['id'];
		}
		return $this->paymentMapping['transfer']['id'];
	}

	private function getDeliveryType($delivery) {
		if (array_key_exists($delivery, $this->deliveryMapping)) {
			return $this->deliveryMapping[$delivery]['id'];
		}
		return $this->deliveryMapping['none']['id'];
	}

	public function hookActionOrderStatusPostUpdate($params) {
		$api = new \iInvoices\Api\ApiClient($this->apiDomain, $this->apikey, $this->apiSecret);

		$state = Configuration::get(self::STATUS_MAP_PREFIX . $params['newOrderStatus']->id);

		$data = array(
				'orderStatus' => $this->getStateType($state)
		);

		$orders = $api->orders->search(array(
				'customId' => self::CUSTOM_ID_PREFIX . $params['id_order']
		));

		$order = current($orders->getItems());

		try {
			$api->orders->update($order->id, $data);
		} catch (\Exception $e) {
			Tools::dieOrLog(sprintf($this->l('Error: minifaktura updateStatus: %s  data: %s'), $e->getMessage(), json_encode($data)), FALSE);
		}
	}

	private function getStateType($state) {
		if (array_key_exists($state, $this->statusMapping)) {
			return $this->statusMapping[$state]['id'];
		}
		return $this->statusMapping['pending_processment']['id'];
	}

	public function hookActionPaymentConfirmation($params) {
		if ($this->invoice !== self::ORDER_INVOICE) {
			return;
		}

		$api = new \iInvoices\Api\ApiClient($this->apiDomain, $this->apikey, $this->apiSecret);

		$data = array(
				'paid' => TRUE
		);

		$invoices = $api->invoices->search(array(
				'customId' => self::CUSTOM_ID_PREFIX . $params['id_order']
		));

		$invoice = current($invoices->getItems());

		try {
			$api->invoices->update($invoice->id, $data);
		} catch (\Exception $e) {
			Tools::dieOrLog(sprintf($this->l('Error: minifaktura PaymentConfirmation: %s  data: %s'), $e->getMessage(), json_encode($data)), FALSE);
		}
	}

}
